#ifndef STRINGPARSER_H____
#define STRINGPARSER_H____

extern long long stringparser(const unsigned char *, long long, long long,
                              const unsigned char **, long long *);

#endif
